#ifndef _BENCHMARK_HH
#define _BENCHMARK_HH 1

#include "mapper.hh"

Read random_extract_read(const Index & i, Index::size_type size) {
  Read r;
  r.errors = 0;
  r.from = rand() % (i.size - size + 1);
  r.data = i.Extract(r.from, size);
  return r;
}

#if 0
void benchmark_mapper(Mapper & mapper, int count) {
  vector<Read> reads;
  reads.reserve(count);

  Timer t;

  for (int i = 0 ; i < count ; ++i) {
    reads.push_back(random_extract_read(mapper.index, mapper.read_size));
    errorify(reads[i], mapper.max_errors);
  }

  cerr << "Random extracted " << count << " of size " << mapper.read_size << " in " << t.msecs() << " msecs\n";

  double avg = 0.;
  int mapped = 0;

  for (int i = 0 ; i < count ; ++i) {
    Mapping m = mapper.map(reads[i]);
    if (m.from == reads[i].from)
      ++mapped;
#if 0
    if (m.from == reads[i].from) {
      cerr << "Mapped with " << m.errors << "/" << reads[i].errors << " errors:\n";
      continue;
    } else
      cerr << "Not mapped (" << m.from << "!=" << reads[i].from << ") with "
        << m.errors << "/" << reads[i].errors << " errors:\n";
    ustring ms = mapper.index.Extract(m.from, mapper.read_size);
    cerr << "  ";
    dna::Unmap(ms);
    cerr.write((char*)ms.str, ms.size);
    cerr << "\n  ";
    dna::Unmap(reads[i].data);
    cerr.write((char*)reads[i].data.str, reads[i].data.size);
    cerr << "\n  ";
    for (int c = 0 ; c < mapper.read_size ; ++c)
      cerr << (reads[i].data.str[c] != ms.str[c] ? "^" : " ");
    cerr << "\n";
#endif
  }

  avg /= count;

  cerr << "Mapped " << mapped << "/" << count << " in " << t.msecs() << " msecs\n";
  double avg_hits = (double)mapper.total_hits / mapper.checked;
  cerr << "Avg hits per slice " << avg_hits << " (" << mapper.total_hits << " / " << mapper.checked << ")\n";
}

void benchmark_random_extract(const Index & index, int size, int count) {
  double avg = 0.;

  Timer t;

  int count_1 = 0;
  int count_10 = 0;
  int count_100 = 0;
  int count_1000 = 0;
  int count_10000 = 0;
  int count_100000 = 0;
  int count_rest = 0;

  for (int i = 0 ; i < count ; ++i) {
    Read r = random_extract_read(index, size);
    int c = index.Count(r.data);
    if (c == 1)
      ++count_1;
    else if (c < 10)
      ++count_10;
    else if (c < 100)
      ++count_100;
    else if (c < 1000)
      ++count_1000;
    else if (c < 10000)
      ++count_10000;
    else if (c < 100000)
      ++count_100000;
    else {
      cerr << "Count " << c << " for " << dna::Unmap(r.data) << "\n";
      ++count_rest;
    }
    avg += c;
  }

  avg /= count;

  cerr << "Counted " << count << " randomly extracted reads of size " << size
    << " with avg " << avg << " hits in " << t.msecs() << " msecs\n";
  cerr << " 1: " << count_1 << "\n";
  cerr << " 1-10: " << count_10 << "\n";
  cerr << " 10-100: " << count_100 << "\n";
  cerr << " 100-1000: " << count_1000 << "\n";
  cerr << " 1000-10000: " << count_10000 << "\n";
  cerr << " 10000-100000: " << count_100000 << "\n";
  cerr << " 100000+: " << count_rest << "\n";
}

void benchmark_random(const Index & index, int size, int count) {
  double avg = 0.;

  Timer t;

  DnaString d(size);

  for (int i = 0 ; i < count ; ++i) {
    avg += index.Count(dna::Randomize(d));
  }

  avg /= count;

  cerr << "Counted " << count << " random reads of size " << size
    << " with avg " << avg << " hits in " << t.msecs() << " msecs\n";
}

void benchmark_mapper(Mapper & mapper, const std::string & what) {
  int count = 1;
  vector<Read> reads;
  reads.resize(count);
  reads[0].data = dna::Map(what);


  Timer t;

  double avg = 0.;
  int mapped = 0;

  for (int i = 0 ; i < count ; ++i) {
    Mapping m = mapper.map(reads[i]);
    if (m.from == reads[i].from)
      ++mapped;
#if 1
    cerr << "Mapped with " << m.errors << "/" << reads[i].errors << " errors:\n";
    Mapper::mstring ms = mapper.index.Extract(m.from, mapper.read_size);
    cerr << "  " << dna::Unmap(ms) << "\n";
    cerr << "  " << dna::Unmap(reads[i].data) << "\n  ";
    for (int c = 0 ; c < mapper.read_size ; ++c)
      cerr << (reads[i].data[c] != ms[c] ? "^" : " ");
    cerr << "\n";
#endif
  }

  avg /= count;

  cerr << "Mapped " << mapped << "/" << count << " in " << t.msecs() << " msecs\n";
  double avg_hits = (double)mapper.total_hits / mapper.checked;
  cerr << "Avg hits per slice " << avg_hits << " (" << mapper.total_hits << " / " << mapper.checked << ")\n";
}

#endif

#endif // _BENCHMARK_HH
