#include <gtest/gtest.h>

#include "bits.hh"

TEST(BitArray, Create)
{
  BitArray<> ba1;
  BitArray<> ba2{10};
  BitArray<> ba3{100};
  BitArray<> ba4{1000};
  BitArray<> ba5{10000};
}

TEST(BitArray, Size)
{
  BitArray<> empty;
  EXPECT_EQ(0, empty.size);
  BitArray<> big{1024};
  EXPECT_EQ(1024, big.size);
}

TEST(BitArray, GetSetReset)
{
  BitArray<> ba{1024};

  auto get = [&ba](size_t n){ return ba[n]; };
  auto notget = [&ba](size_t n){ return !ba[n]; };

  for (auto i = 0u; i < ba.size; ++i) {
    EXPECT_PRED1(notget, i);
  }

  for (auto i = 0u; i < ba.size ; ++i) {
    ba.Set(i);
    EXPECT_PRED1(get, i);
  }

  for (auto i = 0u; i < ba.size; ++i) {
    ba.Reset(i);
    EXPECT_PRED1(notget, i);
  }

  for (auto i = 0u; i < ba.size; ++i) {
    EXPECT_PRED1(notget, i);
    ba.Set(i);
    EXPECT_PRED1(get, i);
    ba.Reset(i);
    EXPECT_PRED1(notget, i);
  }
}

TEST(BitRankSL, Create)
{
  BitRankSL<> ba1;
  BitRankSL<> ba2{10};
  BitRankSL<> ba3{100};
  BitRankSL<> ba4{1000};
  BitRankSL<> ba5{10000};
}

TEST(BitRankSL, Size)
{
  BitRankSL<> empty;
  EXPECT_EQ(0, empty.size);
  BitRankSL<> big{1024};
  EXPECT_EQ(1024, big.size);
}

TEST(BitRankSL, GetSetReset)
{
  BitRankSL<> ba{1024};

  auto get = [&ba](size_t n){ return ba[n]; };
  auto notget = [&ba](size_t n){ return !ba[n]; };

  for (auto i = 0u; i < ba.size; ++i) {
    EXPECT_PRED1(notget, i);
  }

  for (auto i = 0u; i < ba.size ; ++i) {
    ba.Set(i);
    EXPECT_PRED1(get, i);
  }

  for (auto i = 0u; i < ba.size; ++i) {
    ba.Reset(i);
    EXPECT_PRED1(notget, i);
  }

  for (auto i = 0u; i < ba.size; ++i) {
    EXPECT_PRED1(notget, i);
    ba.Set(i);
    EXPECT_PRED1(get, i);
    ba.Reset(i);
    EXPECT_PRED1(notget, i);
  }
}

TEST(BitRankSL, Rank)
{
  BitRankSL<> ba{1000};
  ba.CalculateRanks();

  for (auto i = 0u; i < ba.size; ++i) {
    EXPECT_EQ(0, ba.Rank(i));
    EXPECT_EQ(i + 1, ba.Rank0(i));
  }

  ba.Set(0);
  ba.CalculateRanks();

  for (auto i = 0u; i < ba.size; ++i) {
    EXPECT_EQ(1, ba.Rank(i));
    EXPECT_EQ(i, ba.Rank0(i));
  }

  ba.Set(10);
  ba.Set(12);
  ba.Set(122);
  ba.Set(124);
  ba.Set(128);
  ba.Set(255);
  ba.Set(999);
  ba.CalculateRanks();

  EXPECT_EQ(1, ba.Rank(0));
  EXPECT_EQ(1, ba.Rank(9));
  EXPECT_EQ(2, ba.Rank(10));
  EXPECT_EQ(2, ba.Rank(11));
  EXPECT_EQ(3, ba.Rank(12));
  EXPECT_EQ(3, ba.Rank(121));
  EXPECT_EQ(4, ba.Rank(122));
  EXPECT_EQ(4, ba.Rank(123));
  EXPECT_EQ(5, ba.Rank(124));
  EXPECT_EQ(5, ba.Rank(127));
  EXPECT_EQ(6, ba.Rank(128));
  EXPECT_EQ(6, ba.Rank(254));
  EXPECT_EQ(7, ba.Rank(255));
  EXPECT_EQ(7, ba.Rank(256));
  EXPECT_EQ(7, ba.Rank(998));
  EXPECT_EQ(8, ba.Rank(999));

  for (auto i = 0u; i < ba.size; ++i) {
    ba.Set(i);
  }

  ba.CalculateRanks();

  for (auto i = 0u; i < ba.size; ++i) {
    EXPECT_EQ(i + 1, ba.Rank(i));
    EXPECT_EQ(0, ba.Rank0(i));
  }
}
