#ifndef BWT_HH_
#define BWT_HH_ 1

#include <iostream>

#include <divsufsort64.h>

#include "dna.hh"
#include "util/array.hh"
#include "util/timer.hh"

#include "bwt_base.hh"

template <typename Meta>
class BWT {
 public:
  typedef BWTSiblings<Meta> SiblingTypes;
  typedef typename Meta::size_type size_type;
  typedef typename Meta::string_type string_type;
  typedef typename Meta::compact_type compact_type;

  compact_type bwt;
  size_type end_index;

  BWT() {}
  BWT(size_type n) : bwt(n) {}

 private:
  friend class boost::serialization::access;
  template <typename Archive>
  void serialize(Archive & ar, const unsigned int version);
};

template <typename Meta>
template <typename Archive>
void BWT<Meta>::serialize(Archive & ar, const unsigned int version) {
  ar & bwt;
  ar & end_index;
}

template <typename Meta>
BWT<Meta> BuildBWT(const typename Meta::string_type & text) {
  typedef typename Meta::size_type size_type;
  typedef typename Meta::string_type string_type;
  Timer timer;

  string_type bwt_t(text.size + 1);
  for (size_type i = 0; i < text.size; ++i)
    bwt_t[i] = text[i];

  size_type end_index = divbwt64(bwt_t.data, bwt_t.data, NULL, text.size);

  std::cerr << "[bwt] divbwt() took " << timer.msecs() << " msecs\n";

  for (size_type i = bwt_t.size - 1 ; i > end_index ; --i)
    bwt_t[i] = bwt_t[i - 1];
  bwt_t[end_index] = 0;

  BWT<Meta> bwt(text.size + 1);
  bwt.end_index = end_index;
  for (size_type i = 0; i < bwt_t.size; ++i)
    bwt.bwt.Set(i, bwt_t[i]);

  std::cerr << "[bwt] fix-up took " << timer.msecs() << " msecs\n";

  return bwt;
}

#endif // BWT_HH_
