#ifndef BWT_CUDA_HH_
#define BWT_CUDA_HH_ 1

#include <iostream>

#include "util/array.hh"
#include "util/timer.hh"
#include "util/siblings.hh"
#include "dna.hh"

#include "bwt_base.hh"
#include "bwt.hh"

template <typename Meta>
struct KernelBWT {
  typedef BWTSiblings<Meta> SiblingTypes;
  typedef typename Meta::kernel_compact_type compact_type;
  typedef typename Meta::size_type size_type;

  compact_type bwt;
  size_type end_index;
};

template <typename Meta>
class CudaBWT {
 public:
  typedef BWTSiblings<Meta> SiblingTypes;
  typedef typename Meta::size_type size_type;
  typedef typename Meta::cuda_compact_type compact_type;
  typedef typename SiblingTypes::Host HBWT;
  typedef typename SiblingTypes::Kernel KBWT;

  compact_type bwt;
  size_type end_index;

  CudaBWT(const HBWT & b) : bwt(b.bwt), end_index(b.end_index) { }

  KBWT ForKernel() const { KBWT kb; kb.bwt = bwt.ForKernel(); kb.end_index = end_index; return kb; }
};

#endif // BWT_CUDA_HH_
