#include <string>

#include <gtest/gtest.h>

#include "bwt.hh"
#include "dna.hh"
#include "index.hh"

using namespace bmap;

using std::string;

template<typename Meta, size_t N>
::testing::AssertionResult Check(const char(&text)[N], const char(&expected)[N+1])
{
  typedef typename Meta::string_type string_type;

  string_type t{N - 1};
  std::copy(text, text + N - 1, t.data);
  string_type res{N};
  BuildBWT<Meta>(t).bwt.Extract(0, N, res);

  if (res.size != N || memcmp(res.data, expected, N) != 0)
    return ::testing::AssertionFailure() << "'" << string(expected, N) << "' != '" << res << "'";

  return ::testing::AssertionSuccess();
}


TEST(BWT, BuildMisc)
{
  EXPECT_TRUE(Check<CharMeta>("111", "111\0"));
  EXPECT_TRUE(Check<CharMeta>("\0\0\0", "\0\0\0\0"));
  EXPECT_TRUE(Check<CharMeta>("\1\1\1", "\1\1\1\0"));
  EXPECT_TRUE(Check<CharMeta>("123456", "6\00012345"));
  EXPECT_TRUE(Check<CharMeta>("ROWER", "RWRE\0O"));
  EXPECT_TRUE(Check<CharMeta>("ABAAC", "CB\0AAA"));
}

TEST(BWT, BuildDna)
{
  EXPECT_TRUE(Check<DnaMeta>("\0\0\0", "\0\0\0\0"));
  EXPECT_TRUE(Check<DnaMeta>("\1\1\1", "\1\1\1\0"));
  EXPECT_TRUE(Check<DnaMeta>("\0\1\2\3", "\3\0\0\1\2"));
}
