#ifndef _INDEX_API_HH
#define _INDEX_API_HH

#include <cuda_runtime.h>

#include "util/array.hh"
#include "util/siblings.hh"

#include "ssa_cuda.hh"
#include "index_base.hh"

// API isolating from the CUDA kernels
template <typename Index>
struct CudaIndexAPI {
  typedef typename CSibling<Index>::Type CudaIndex;
  typedef typename KSibling<Index>::Type KernelIndex;

  typedef typename Index::string_type read_type;
  typedef SArray<read_type> read_array;

  typedef DnaStrings strings_type;
  typedef CSibling<DnaStrings>::Type cuda_strings_type;
  typedef KSibling<DnaStrings>::Type kernel_strings_type;

  typedef SArray<typename Index::Result> result_array;
  typedef CudaArray<typename Index::Result> cuda_result_array;
  typedef KernelArray<typename Index::Result> kernel_result_array;

  typedef typename Index::size_type size_type;
  typedef SArray<size_type> size_array;
  typedef CudaArray<size_type> cuda_size_array;
  typedef KernelArray<size_type> kernel_size_array;

  CudaIndexAPI(const Index & index) : cuda_index(index) { }

  void find(const cuda_strings_type & creads, const cuda_size_array & cread_ids, size_type offset, size_type len, int err_pos, int err, cuda_result_array & cresults) const;
  void index_to_text(const size_array & index_pos, size_array & text_pos) const;
  void index_to_text(const cuda_size_array & index_pos, cuda_size_array & text_pos) const;

  void compare_to_text(const cuda_strings_type & creads, const cuda_size_array & cread_map, const cuda_size_array & ctext_pos, int offset, int length, cuda_size_array & cerrors) const;

  CudaIndex cuda_index;
};

#endif // _INDEX_API_HH
