#ifndef _SSA_BASE_HH_
#define _SSA_BASE_HH_

#include "util/siblings.hh"

template <typename Meta, typename SSAImp>
struct SSABase;

template <typename Meta, typename SSAImp>
struct CudaSSABase;

template <typename Meta, typename SSAImp>
struct KernelSSABase;

template <typename Meta>
struct SSA;

template <typename Meta>
struct CudaSSA;

template <typename Meta>
struct KernelSSA;

template <typename Meta>
struct SSAT;

template <typename Meta>
struct CudaSSAT;

template <typename Meta>
struct KernelSSAT;

template <typename Meta>
struct SSATS;

template <typename Meta>
struct SSASiblings {
  typedef SSA<Meta> Host;
  typedef CudaSSA<Meta> Cuda;
  typedef KernelSSA<Meta> Kernel;
};

template <typename Meta>
struct SSATSiblings {
  typedef SSAT<Meta> Host;
  typedef CudaSSAT<Meta> Cuda;
  typedef KernelSSAT<Meta> Kernel;
};

template <typename Meta, typename SSAImpl>
struct SSABaseSiblings {
  typedef typename HSibling<SSAImpl>::Type HostImpl;
  typedef typename CSibling<SSAImpl>::Type CudaImpl;
  typedef typename KSibling<SSAImpl>::Type KernelImpl;
  typedef SSABase<Meta, HostImpl> Host;
  typedef CudaSSABase<Meta, CudaImpl> Cuda;
  typedef KernelSSABase<Meta, KernelImpl> Kernel;
};

// The lot below needs to be specialized like that b/c of CRTP causing incomplete types

template <typename Meta>
struct Siblings<SSA<Meta> > {
  typedef SSASiblings<Meta> Types;
};

template <typename Meta>
struct Siblings<CudaSSA<Meta> > {
  typedef SSASiblings<Meta> Types;
};

template <typename Meta>
struct Siblings<KernelSSA<Meta> > {
  typedef SSASiblings<Meta> Types;
};

template <typename Meta>
struct Siblings<SSAT<Meta> > {
  typedef SSATSiblings<Meta> Types;
};

template <typename Meta>
struct Siblings<CudaSSAT<Meta> > {
  typedef SSATSiblings<Meta> Types;
};

template <typename Meta>
struct Siblings<KernelSSAT<Meta> > {
  typedef SSATSiblings<Meta> Types;
};

#endif // _SSA_BASE_HH_
