#include <gtest/gtest.h>

#include "bits.hh"
#include "util/compact.hh"

typedef CompactArray<uint64_t, 2, uint8_t> CA;

TEST(CompactArray, Create)
{
  CA ca1;
  CA ca2{10};
  CA ca3{100};
  CA ca4{1000};
  CA ca5{10000};
}

TEST(CompactArray, Size)
{
  CA empty;
  EXPECT_EQ(0, empty.size);
  CA big{1024};
  EXPECT_EQ(1024, big.size);
}

TEST(CompactArray, SetGet)
{
  CA ca{1024};

  auto check = [&ca](size_t n, uint8_t what){ return ca[n] == what; };

  for (auto i = 0u; i < ca.size ; ++i) {
    ca.Set(i, i % 4);
    EXPECT_PRED2(check, i, i % 4);
  }

  for (auto i = 0u; i < ca.size ; ++i) {
    ca.Set(i, (i + 1) % 4);
    EXPECT_PRED2(check, i, (i + 1) % 4);
  }
}

TEST(CompactArray, FromSArray)
{
  SArray<uint8_t> sa(128);

  for (auto i = 0u; i < sa.size ; ++i)
    sa[i] = i % 4;

  CA ca(sa);
  EXPECT_EQ(sa.size, ca.size);

  auto check = [&ca](size_t n, uint8_t what){ return ca[n] == what; };

  for (auto i = 0u; i < ca.size ; ++i)
    EXPECT_PRED2(check, i, i % 4);
}

TEST(CompactArray, ToSArray)
{
  CA ca{1024};

  for (auto i = 0u; i < ca.size ; ++i)
    ca.Set(i, i % 4);

  SArray<uint8_t> sa(128);

  auto check = [&sa](size_t n, uint8_t what){ return sa[n] == what; };

  ca.Extract(0, 128, sa);

  for (auto i = 0u; i < sa.size ; ++i)
    EXPECT_PRED2(check, i, i % 4);

  ca.Extract(1, 128, sa);

  for (auto i = 0u; i < sa.size ; ++i)
    EXPECT_PRED2(check, i, (i + 1) % 4);
}

