#ifndef _UTIL_PACKED_HH_
#define _UTIL_PACKED_HH_

#include <cuda_runtime.h>

//#define USE_UNION

template <int size, typename Base, int ElementSize, typename Element>
class PackedArray {
 public:
  PackedArray(){};
  PackedArray(const SArray<Element> & array);

  const static size_t BaseSize = sizeof(Base) * 8 / ElementSize;
  const static Base ElemMask = (1ULL << ElementSize) - 1;

  __device__ __host__ Element Get(size_t n) const;
  void Set(size_t n, Base el);

  __device__ __host__ Element operator[](size_t n) const;

#ifdef USE_UNION
  union {
    ulonglong2 vector;
    uint64_t data[2];
  } data;
#else
  ulonglong2 vector;
#endif
};

template <int size, typename Base, int ElementSize, typename Element>
PackedArray<size, Base, ElementSize, Element>::PackedArray(const SArray<Element> & a) {
  for (int i = 0; i < size; ++i)
      Set(i, a[i]);
}

template <int size, typename Base, int ElementSize, typename Element>
Element PackedArray<size, Base, ElementSize, Element>::Get(size_t n) const {
#ifdef USE_UNION
  return (data.data[n / BaseSize] >> (ElementSize * (n % BaseSize))) & ElemMask;
#else
  Base data = n / BaseSize ? vector.y : vector.x;
  return (data >> (ElementSize * (n % BaseSize))) & ElemMask;
#endif
}

template <int size, typename Base, int ElementSize, typename Element>
void PackedArray<size, Base, ElementSize, Element>::Set(size_t n, Base val) {
  int offset = ElementSize * (n % BaseSize);
  Base *data = reinterpret_cast<Base *>(&vector);
  data[n / BaseSize] &= ~(ElemMask << offset);
  data[n / BaseSize] |= val << offset;
}
#undef USE_UNION

template <int size, typename Base, int ElementSize, typename Element>
Element PackedArray<size, Base, ElementSize, Element>::operator[](size_t n) const {
  return Get(n);
}

#endif // _UTIL_PACKED_HH_
