#ifndef _UTIL_TIMER_HH_
#define _UTIL_TIMER_HH_

#include <ctime>
#include <sys/time.h>

class Timer {
 public:
  Timer() : last(clock()), total(0) {
  }

  int msecs() {
    clock_t diff = clock() - last;
    last += diff;
    total += diff;
    return diff / (CLOCKS_PER_SEC / 1000);
  }

  int total_msecs() {
    msecs();
    return total / (CLOCKS_PER_SEC / 1000);
  }

 private:
  clock_t last;
  clock_t total;
};

class CpuTimer {
 public:
  CpuTimer() : total_ms(0) {
    clock_gettime(CLOCK_THREAD_CPUTIME_ID, &last);
  }

  int msecs() {
    timespec newt;
    int diff_ms;
    clock_gettime(CLOCK_THREAD_CPUTIME_ID, &newt);
    diff_ms = (newt.tv_sec - last.tv_sec) * 1000;
    diff_ms += (newt.tv_nsec - last.tv_nsec) / 1000000L;
    total_ms += diff_ms;
    last = newt;
    return diff_ms;
  }

  int total_msecs() {
    msecs();
    return total_ms;
  }

 private:
  timespec last;
  int total_ms;
};



class RealTimer {
 public:
  RealTimer() : total_ms(0) {
    gettimeofday(&last, NULL);
  }

  int msecs() {
    timeval newt;
    int diff_ms;
    gettimeofday(&newt, NULL);
    diff_ms = (newt.tv_sec - last.tv_sec) * 1000;
    diff_ms += (newt.tv_usec - last.tv_usec) / 1000L;
    total_ms += diff_ms;
    last = newt;
    return diff_ms;
  }

  int total_msecs() {
    msecs();
    return total_ms;
  }

 private:
  timeval last;
  int total_ms;
};

#endif // _UTIL_TIMER_HH_
